require 'extensions.rb'
require 'LangHandler.rb'
require '+++/fr_quick_selection.rb'
require '+++/fr_geom_selection.rb'
require '+++/entfilter.rb'
require 'sketchup.rb'

#**************************************************************
#Dplacer sur l'origine
class Move2Origin

def initialize 

  bb = Geom::BoundingBox.new

  am = Sketchup.active_model        # Sketchup Active Model. 
  am.start_operation "Move To Origin" ; 
  se = am.active_entities           # Sketchup Active Entities.  Allows working inside a component.
  ss = am.selection                 # Work with selection, if any... 
  if (ss.length == 0) then ss = se ; end ;     # ...else work with whole model. 

  ss.each {|e|                       # add all geometry to our temporary bounding box
    bb.add(e.bounds) ; 
    } 
 
  trans = Geom::Transformation.new(bb.corner(0).vector_to(ORIGIN)) ;  # move the temp bounding box 

  status = Sketchup.active_model.entities.transform_entities(trans, ss.to_a ) ; 
  #puts "status is #{status}" ; 

  am.commit_operation ; 
  end ;  # initialize 

end ; # class 

#**************************************************************
#Isoler selection
class Hide
	def Hide.hideAll
		model=Sketchup.active_model
		model.start_operation "Isoler selection"
		ents=model.active_entities
		ss=model.selection
		if ss.empty?
			UI.messagebox("Aucune selection.")
			return nil
		else
			ents.each{|e|e.hidden=true}
			ss.each{|x|x.hidden=false}
			model.commit_operation
		end
	end
end

#**************************************************************
#Inverser slection
def se_inv
    model = Sketchup.active_model
    ss = model.selection
	if ss.empty?
		UI.messagebox("Aucune selection.")
		return nil
	else
		model.entities.each {|e| ss.toggle(e)}
	end
end

#Zoom sur slection
def zent
    model=Sketchup.active_model
	sel=model.selection
	if sel.empty?
		UI.messagebox("Aucune selection.")
		return nil
		else
			Sketchup.send_action("viewZoomToSelection:")
	end
end

#**************************************************************
#Annuler toutes les texturisations
def deltxt
	model=Sketchup.active_model
	ents=model.active_entities
	ents.each do |e|
		type = e.typename
		case type
			when "Face"
				e.material = nil if e.material !=nil
				e.back_material = nil if e.back_material !=nil
			when "Group"
				ents = e.entities
				ents.each do |gre|
					if gre.typename == "Face"
						gre.material = nil if gre.material !=nil
						gre.back_material = nil if gre.back_material !=nil
					end
				end
			when "ComponentInstance"
				ents = e.definition.entities
				ents.each do |ce|
					if ce.typename == "Face"
						ce.material = nil if ce.material !=nil
						ce.back_material = nil if ce.back_material !=nil
					end
				end
		end
	end
	UI.messagebox("Toutes les textures ont t annules.")
	model.commit_operation
end

#**************************************************************
#Texturer la slection
def seltxt
	model = Sketchup.active_model
	if model.materials.current !=nil
		active_material = model.materials.current.name
		puts "Texture active = " + active_material
		if model.selection.empty?
			UI.messagebox("Aucune slection.")
			return nil
		else
			ents = model.selection
			ents.each do |e|
				case e.typename
					when "Face"
						avant = e.material
						arriere = e.back_material
						e.material = model.materials.current if avant == nil or avant != active_material
						e.back_material = model.materials.current if arriere == nil or arriere != active_material
					when "Group"
						ents = e.entities
						ents.each do |gre|
							if gre.typename == "Face"
								avant = gre.material
								arriere = gre.back_material
								gre.material = model.materials.current if avant == nil or avant != active_material
								gre.back_material = model.materials.current if arriere == nil or arriere != active_material
							end
						end
					when "ComponentInstance"
						ents = e.definition.entities
						ents.each do |ce|
							if ce.typename == "Face"
								avant = ce.material
								arriere = ce.back_material
								ce.material = model.materials.current if avant == nil or avant != active_material
								ce.back_material = model.materials.current if arriere == nil or arriere != active_material
							end
						end
				end
			end
			puts "Texturisation de la slection termine."
		end
	else
		UI.messagebox("Aucune texture active.")
		return nil
	end
	model.commit_operation
end

#**************************************************************
#Afficher boite de dialogue nouveau...
def new_from_template
   #template_dir = Sketchup.find_support_file "Templates"
   template_dir = Sketchup.template_dir
   files = Dir[File.join(template_dir, "*.skp")].sort
   f = UI.inputbox(["Template:"], [""], [files.map{|f| File.basename(f)}.join("|")], "New From Template")
   return unless f
   t = File.join(template_dir, f[0])
   t = "" if t.chop == template_dir
   Sketchup.template = t
   Sketchup.file_new
end

#**************************************************************
#Reprsenter l'angle du soleil pour un meilleur placement des ombres
class ShadowInfoObserver < Sketchup::ShadowInfoObserver

  def register(tool)
    @tool = tool
  end

  def onShadowInfoChanged(*args)
    si = args[0]
    if @tool
      @tool.update(si)
    else
      raise "#{$0}Tool not registered."
    end
    if $DEBUG
      print "onShadowInfoChanged: "
      p args
      puts "sun_direction=#{sun_direction.inspect}"
      Sketchup.active_model.entities.add_cpoint ORIGIN + v
    end
  end

end # class ShadowInfoObserver

class SunTool

  # called when the ShadowInfo is changed
  def update(si)
    if @DEBUG
      print "update: "
      p si 
    end
    v = si["SunDirection"]
    v.length = 100.feet
    t = si["ShadowTime"]
    tz = si["TZOffset"]
    t = t - (60 * 60 * tz)
    ft = t.strftime("%m/%d/%Y %H:%M %p")
    Sketchup.set_status_text("Shadow Time: #{ft}", SB_PROMPT)
    @pt = ORIGIN + v
    Sketchup.active_model.active_view.invalidate
  end
  def activate
    @shadow_observer = ShadowInfoObserver.new
    @shadow_observer.register(self)
    Sketchup.active_model.shadow_info.add_observer(@shadow_observer)
    Sketchup.set_status_text "Sun Tool", SB_PROMPT
  end
  def deactivate(*args)
    if $DEBUG
      print "deactivate: "
      p args
    end
    Sketchup.active_model.shadow_info.remove_observer(@shadow_observer)
  end

  def resume(view)
    Sketchup.set_status_text "Sun Tool", SB_PROMPT
    view.invalidate
  end


  def draw(view)
    if $DEBUG
      print "draw: "
      p view
      p @pt
    end
    if @pt
      view.draw_points([@pt], 25, 1, 2)
      view.draw_line(ORIGIN, @pt)
    end
  end
end

#**************************************************************
#Reprsenter la course du soleil
def map_sun

  # User variables
  tz = -5.0
  altitude = 5.m

  Sketchup.active_model.start_operation "Map Sun"
  si = Sketchup.active_model.shadow_info
  ct = si['ShadowTime']
  yr = ct.year
  mon = ct.month
  day = ct.day
  e = Sketchup.active_model.entities.add_group
  e.name  = "#{mon}-#{day}"
  0.upto(23) { |h|
    t = Time.local(yr, mon, day, h, 0)
    si['ShadowTime'] = t
    v = si['SunDirection']
    v.length = altitude
    pt = ORIGIN + v
    e.entities.add_cpoint(pt)
    e.entities.add_text(((h-tz) % 24).to_i.to_s, pt)
  }
  si['ShadowTime'] = ct
  Sketchup.active_model.commit_operation
end

#**************************************************************
#Supprimer les elts non utiliss
def pur
model=Sketchup.active_model

cd=model.definitions.count
model.definitions.purge_unused
cd=cd-model.definitions.count

cl=model.layers.count
model.layers.purge_unused
cl=cl-model.layers.count

cm=model.materials.count
model.materials.purge_unused
cm=cm-model.materials.count

cs=model.styles.count
model.styles.purge_unused
cs=cs-model.styles.count

UI.messagebox("Purges:\n\nComposants :\t "+cd.to_s+" lments.\nCalques :\t\t "+cl.to_s+" lments.\nMatriaux :\t "+cm.to_s+" lments.\nStyles :\t\t "+cs.to_s+" lments.\t")

model.commit_operation
end