# RubyPanel Toolbar (C) 2007 jim.foltz@gmail.com
#
# With special thanks to Chris Phillips (Sketchy Physics)
# for the Win32API code examples. 


# I wrap everything in my module. It reduces
# the likelyhood of naming clashes.
module JF
	require "sketchup.rb"
      
   case Sketchup.os_language
   when "FR"
      @wname = "Console Ruby"
   when "EN"
   else
      @wname = "Ruby Console"
   end

   @rect = Sketchup.read_default("RubyToolbar", "Position")
   unless @rect.nil?
      @rect = @rect.split(",").map{|e| e.to_i}
   end

   def self.findWindow 
      Win32API.new("user32.dll", "FindWindow", ['P','P'], 'N')
   end

   def self.sendMessage 
      Win32API.new("user32.dll", "SendMessage", ['N','N','N','P'], 'N')
   end

   def self.getConsole
      return findWindow.call(0, @wname)
   end

   def self.consoleOpen?
      return (getConsole() != 0)
   end
   def self.consoleClosed?
      return (not consoleOpen?)
   end


   def self.closeConsole
      pw = getConsole()
      sendMessage.call(pw,0x0010,0,"")
   end 

   def self.openConsole
      res = Sketchup.send_action "showRubyPanel:"
      if @rect
	 UI.start_timer(0.5) { moveConsole(@rect) }
      end
   end

   def self.clearConsole
      moveConsole(@rect)
      findWindowEx = Win32API.new("user32.dll", "FindWindowEx", ['N','N','P','P'], 'N')
      pw = getConsole
      h=findWindowEx.call(pw,0,"Edit",0) 
      h=findWindowEx.call(pw,h,"Edit",0) 
      sendMessage.call(h,0x000C,0,"")			
      puts ""
   end



   def self.moveConsole(rect)
      x, y, w, h = rect
      setWindowPos= Win32API.new("user32.dll", "SetWindowPos", ['P','P','N','N','N','N','N'], 'N') 
      pw=findWindow.call(0,"Ruby Console")
      pw = getConsole
      setWindowPos.call(pw,0,x,y,w,h,0)
   end 

   def self.getConsoleLocation
      getWindowRect= Win32API.new("user32.dll", "GetWindowRect",['P','PP'],'N')
      pw = getConsole
      rect=Array.new.fill(0.chr,0..4*4).join
      getWindowRect.call(pw,rect);
      rect=rect.unpack("i*")

      rect[2]=rect[2]-rect[0]
      rect[3]=rect[3]-rect[1]
      return rect
   end 

   def self.toggle
      if consoleClosed?
	 openConsole
      else
	 @rect = getConsoleLocation
	 closeConsole
	 Sketchup.write_default("RubyToolbar", "Position", @rect.join(","))
      end
   end

   def self.rt_load
      sd = Sketchup.find_support_file("Plugins") + "/"
      sd.gsub!("/", "\\\\")
      f = UI.openpanel("Load Script", sd, "*.rb*")
      return unless f
      begin
	 load f
	 Sketchup.set_status_text "#{File.basename(f)} loaded. (#{Time.now.strftime('%H:%M:%S')})"
	 @recent = f
      rescue
	 UI.messagebox("Couldn't load #{File.basename(f)}: #{$!}")
      end
   end

   def self.rt_reload
      if @recent.nil?
	 rt_load
      else
	 load @recent
	 Sketchup.set_status_text "#{File.basename(@recent)} Reloaded. "+Time.now.strftime("%H:%M:%S")
      end
   end

   unless file_loaded?("rb_tools.rb")

	plugins = Sketchup.find_support_file("Plugins")
    imgdir = File.join(plugins, "rubytoolbar")
    tb = UI::Toolbar.new("Console ruby")

	cmd1 = UI::Command.new("Show/Hide") { Sketchup.send_action "showRubyPanel:" }
		cmd1.small_icon = "rb_panel16.png"
		cmd1.large_icon = "rb_panel24.png"
		cmd1.tooltip = cmd1.status_bar_text = "Montrer/Cacher la console"
		tb.add_item(cmd1)
		
	# cmd2 = UI::Command.new("Purger") { clearConsole }
		# cmd2.small_icon = "rb_delete16.png"
		# cmd2.large_icon = "rb_delete24.png"
		# cmd2.tooltip = cmd2.status_bar_text = "Purger la console"
		# tb.add_item(cmd2)

	cmd3 = UI::Command.new("Charger") { rt_load }
		cmd3.small_icon = "rb_load16.png"
		cmd3.large_icon = "rb_load24.png"
		cmd3.tooltip = cmd3.status_bar_text = "Charger"
		tb.add_item(cmd3)
		
	cmd4 = UI::Command.new("Recharger") { rt_reload }
		cmd4.small_icon = "rb_reload16.png"
		cmd4.large_icon = "rb_reload24.png"
		cmd4.tooltip = cmd4.status_bar_text = "Recharger"
		tb.add_item(cmd4)
		
	cmd5 = UI::Command.new("Plugins") { UI.openURL(Sketchup.find_support_file("Plugins")) }
		cmd5.small_icon = "rb_pluginsfolder16.png"
		cmd5.large_icon = "rb_pluginsfolder24.png"
		cmd5.tooltip = cmd5.status_bar_text = "Explorer le dossier plugins"
		tb.add_item(cmd5)

    tb.show if tb.get_last_state == -1

    file_loaded("rb_Tools.rb")

   end

end
